#!/usr/bin/env python
#
# Copyright (C) 2013 S. Daniel Francis <francis@sugarlabs.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
# MA 02110-1301, USA.

import gtk
import logging
import subprocess
from plugins.plugin import Plugin
from gettext import gettext as _
from TurtleArt.tapalette import make_palette
from gps import *
import time
from math import radians, cos, sin, asin, sqrt
import threading
import os
from os.path import join
from TurtleArt.tatype import TYPE_FLOAT
from TurtleArt.tapalette import palette_name_to_index
from TurtleArt.tapalette import special_block_colors
from TurtleArt.tapalette import palette_blocks
from TurtleArt.talogo import logoerror
from TurtleArt.tawindow import block_names
from TurtleArt.taconstants import CONSTANTS
from TurtleArt.taprimitive import Primitive, ArgSlot, ConstantArg, or_
from TurtleArt.tatype import TYPE_INT, TYPE_STRING, TYPE_NUMBER

class Gpsa(Plugin):
	def __init__(self, parent):
		self.tw = parent
		self.frecuencia_medicion = 5
		self._status = False
		self.gpsd=None
		self.d=None
		self.m=None
		self.latitud_anterior=0
		self.longitud_anterior=0
		self.medida_actual=0
		self.ruta_gpsd='plugins/gpsa/'
		self.ejecutarmedida=True
		self.refrescargpad=True
	
	def setup(self):
	
		palette = make_palette('gpss', colors=["#FF00FF", "#A000A0"], help_string=_('GPS Butia'))
		
		palette.add_block('initgps',
			style='basic-style',
			label=_('inicializar gps'),
			prim_name='initgps',
			help_string=_('inicializa el gps'))
		
		self.tw.lc.def_prim('initgps', 0,Primitive(self.inicializar))
		
		palette.add_block('medir',
			style='basic-style-1arg',
			label=_('Comenzar a medir'),
			default=5,
			help_string=_('Setea el periodo de tiempo en el cual se tomaran las medidas (en segundos)'),
			prim_name='medir')
		
		self.tw.lc.def_prim('medir', 1,Primitive(self.medir, arg_descs=or_([ArgSlot(TYPE_NUMBER)], [ArgSlot(TYPE_STRING)])))
		
		palette.add_block('get_latitude',
			style='box-style',
			label=_('obtener latitud'),
			help_string=_('retorna la latitud'),
			value_block=True,
			prim_name='get_latitude')
		
		self.tw.lc.def_prim('get_latitude', 0,Primitive(self.get_latitude, TYPE_STRING))
		
		palette.add_block('get_longitude',
			style='box-style',
			label=_('obtener longitud'),
			help_string=_('retorna la longitud'),
			value_block=True,
			prim_name='get_longitude')

		self.tw.lc.def_prim('get_longitude', 0,Primitive(self.get_longitude, TYPE_STRING))
		
		palette.add_block('get_medida',
			style='box-style',
			label=_('obtener medida'),
			help_string=_('retorna la medida'),
			value_block=True,
			prim_name='get_medida')

		self.tw.lc.def_prim('get_medida', 0,Primitive(self.get_medida, TYPE_STRING))
		
		
		
		palette.add_block('ejecutar_ejemplo',
			style='box-style',
			label=_('ejecutar ejemplo'),
			help_string=_('retorna la distancia recorrida en el archivo de ejemplo'),
			value_block=True,
			prim_name='ejecutar_ejemplo')

		self.tw.lc.def_prim('ejecutar_ejemplo', 0,Primitive(self.ejecutar_ejemplo, TYPE_STRING))
			
		palette.add_block('fingps',
			style='basic-style',
			label=_('finalizar gps'),
			prim_name='fingps',
			help_string=_('finaliza el gps'))
		
		self.tw.lc.def_prim('fingps', 0,Primitive(self.finalizar))
		
	def inicializar(self):
			try:
				if (self.gpsd==None):
					terminar=False
					i=0
					while ((terminar==False)and(i<10)):
						valor=chr(ord('0')+i)
						i=i+1
						terminar=self.intentar_inicializar(valor)
					if (terminar):
						self.gpsd = gps(mode=WATCH_ENABLE)
						self.d = threading.Thread(target=self.refrescar, name='refrescar')
						self.d.daemon = True
						self.d.start()
			except:
				print _('error al inicializar')
	def medir(self, x):
		try:
			self.frecuencia_medicion = x
			self.latitud_anterior=self.gpsd.fix.latitude
			self.longitud_anterior=self.gpsd.fix.longitude
			self.medida_actual=0
			self.m = threading.Thread(target=self.calcular_medida, name='Calcular_medida')
			self.m.daemon = True
			self.m.start()
		except:
			print _('Error en medir')
			
	def get_latitude(self):
		try:
			return self.gpsd.fix.latitude
		except:
			print _('Error en obtener latitud')
			
	def get_longitude(self):
		try:
			return self.gpsd.fix.longitude
		except:
			print _('Error en obtener longitud')
			
	def refrescar(self):
		try:
			while (self.refrescargpad):
				self.gpsd.next()
				time.sleep(2)

		except:
			print _('Error en obtener latitud')
			
	def get_medida(self):
		return self.medida_actual
	
	def calcular_medida(self):
		try:
			while (self.ejecutarmedida):
				nueva_lat=self.gpsd.fix.latitude
				nueva_long=self.gpsd.fix.longitude
				nueva_lat_f=float(nueva_lat)
				nueva_long_f=float(nueva_long)
				if ((nueva_lat!=0) and (nueva_long!=0) and ((nueva_lat!=self.latitud_anterior) or (nueva_long!=self.longitud_anterior))):
					f = open ("fichero.txt", "a")
					f.write(str(nueva_lat_f)+"  "+str(nueva_long_f)+"\n")
					f.close()
					lat_ant=self.latitud_anterior
					long_ant=self.longitud_anterior
					lat_ant_f=float(lat_ant)
					long_ant_f=float(long_ant)
					self.medida_actual=self.medida_actual+self.haversine(nueva_long_f,nueva_lat_f,long_ant_f,lat_ant_f)
					self.latitud_anterior=nueva_lat
					self.longitud_anterior=nueva_long
				time.sleep(self.frecuencia_medicion)
		except:
			print _('Error en obtener medir')
			
	def haversine(self, lon1, lat1, lon2, lat2):
		# convertir grados decimales a radianes 
		lon1, lat1, lon2, lat2 = map(radians, [lon1, lat1, lon2, lat2])
		# haversine formula 
		dlon = lon2 - lon1 
		dlat = lat2 - lat1 
		a = sin(dlat/2)**2 + cos(lat1) * cos(lat2) * sin(dlon/2)**2
		c = 2 * asin(sqrt(a)) 
		# 6367 km is the radius of the Earth
		medida = 6367 *1000* c
		return medida 

		
	def ejecutar_ejemplo(self):
		distancia=0
		f = open ("Ejemplo.txt", "r")
		linea=f.readline()
		lista=linea.split(' ')
		lat_ant=float(lista[0])
		long_ant=float(lista[1])
		linea=f.readline()
		while (linea!="FIN"):
			lista=linea.split(' ')
			lat_nueva=float(lista[0])
			long_nueva=float(lista[1])
			a=self.haversine(long_ant, lat_ant, long_nueva, lat_nueva)
			distancia=distancia+a
			lat_ant=lat_nueva
			long_ant=long_nueva
			linea=f.readline()
		f.close()
		return distancia
			
	def intentar_inicializar(self,num):
		try:
			str2='chmod 777 plugins/gpsa/gpsd'
			process2 = os.popen(str2)
			str2=self.ruta_gpsd+'./gpsd /dev/ttyUSB0'
			p2=os.popen(str2)
			return True
		except:
			print _('Error al iniciar')
	
	def finalizar(self):
		try:
			self.ejecutarmedida=False
			self.refrescargpad=False
		except:
			print _('Error al finalizar gpsd')
